#ifndef RAWMILLISLICEINDEXHEADER_H
#define RAWMILLISLICEINDEXHEADER_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/Macros.h"

#include "DAQDataFormats/DataFormatFirst.inc"
#define CURRENT_CLASS                RawMilliSliceIndexHeader
#define CURRENT_FIELDS_SUB_NAMESPACE millisliceindexheader

namespace daqdataformats {


    namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t  RAWMILLISLICEINDEXHEADER_SIZE = 1; ///< MilliSliceIndexHeader size in words

            enum MilliSliceIndexHeaderMASKS {
                MASK_Version     = 0x00000FFF, ///< Version Mask                  (12 bits)
                MASK_Closed      = 0x00008000, ///< Is the MilliSliceIndex Closed (1 bit)
                MASK_IndexNumber = 0xFFFF0000  ///< Number of indeces             (16 bits)
	    };// end of MilliSliceIndexHeaderMASKS

            enum MilliSliceIndexHeaderWORDS {
                WORD_Version     = 0, ///< Version Mask                  (12 bits)
                WORD_Closed      = 0, ///< Is the MilliSliceIndex Closed (1 bit)
                WORD_IndexNumber = 0  ///< Number of indeces             (16 bits)
	    };// end of MilliSliceIndexHeaderMASKS

            DEFINE_SHIFT_FROM_MASK(Version)
            DEFINE_SHIFT_FROM_MASK(Closed)
            DEFINE_SHIFT_FROM_MASK(IndexNumber)

    }

    class RawMilliSliceIndexHeader: public RawDAQData{
        public:
            // Constructors
            RawMilliSliceIndexHeader();//! The default constructor
            RawMilliSliceIndexHeader(const RawMilliSliceIndexHeader& copy_in);  //! Deep copy constructor
            RawMilliSliceIndexHeader(const void* buffer);
            /// Destructor
            ~RawMilliSliceIndexHeader();

            uint32_t sizeofdata      () const {return CURRENT_FIELDS_SUB_NAMESPACE::RAWMILLISLICEINDEXHEADER_SIZE;}

            IMPLEMENT_GET_NOVERSION          (Version, uint32_t)
            IMPLEMENT_SET_NOVERSION_PROTECTED(Version, uint32_t)

            IMPLEMENT_IS_AND_SET_NOVERSION   (Closed)
            IMPLEMENT_GET_AND_SET_NOVERSION  (IndexNumber, uint32_t)

            bool    advanceIndexNumber() {return setIndexNumber(getIndexNumber() + 1);}
            
        private:

            version_t figureOutTheVersion(const void*) const {return 0;}
            bool      setFunctionPointers()                  {return true;}


    };
} // END NAMESPCE daqdataformats
#endif /* RAWMILLISLICEINDEXHEADER_H */
