#ifndef RAWNANOSLICEV1_H
#define RAWNANOSLICEV1_H
#include "DAQDataFormats/RawNanoSliceV0.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSlice
#define  CURRENT_CLASS_VERSION        1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawnano
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

  namespace  VERSION_NAMESPACE {
        namespace CURRENT_FIELDS_SUB_NAMESPACE {
            
            enum NanoSliceWORDS{
                                WORD_Value1 = 2
            };// end of NanoSliceMASKS

            enum NanoSliceMASKS{
                                MASK_Value1 = 0xFFFF0000
            };// end of NanoSliceMASKS

            DEFINE_SHIFT_FROM_MASK(Value1);
        };// end of namespace rawnano

  //! Class to hold the data from the FEBs in correct formats
  /*!
    The RawNanoSlice class is designed to mimic the format of the actual data
    block that the NOvA Front End6
    Boards (FEBS) produce when they detect a signal
    above the zero suppression threshold.
    
    The class supports general initialization with or with out data blocks as
    well as copyin style initialization for use with STL containers.
    
    Pack and unpack methods for the data rely on two types of structured data
    blocks that are provided in association with the class.
    
    The NanoSliceBlock structure breaks the raw data block out into its related
    high and low half words to give the headers, time stamps and pulseheight information
    
    The NanoSliceHeader structure further breaks out the header blocks into the
    individual data information parts.
  */

  class RawNanoSlice: public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
      
            // Constructors
            RawNanoSlice (const version_t );
            RawNanoSlice (const NanoSlice&);
            RawNanoSlice (const RawNanoSlice&);
            /// Destructor
            ~RawNanoSlice();

            /// Initialization of the Data members
            void     init();
            void     initWithData(const NanoSlice&);
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print(std::ostream& os=std::cout) const;
            
            /// Get Reserved is not allowed for version 1, because there is no Reserved field
            uint32_t getReserved() const              { GENERATE_FUNCTION_IS_NOT_ALLOWED; return 0;}
            /// Set for Reserved word is not allowed for version 1, because there is no Reserved field
            bool     setReserved   (const uint32_t)   { return GENERATE_FUNCTION_IS_NOT_ALLOWED;}
            /// Get Pulse Height for version > 0 == Get Value1
            uint32_t getPulseHeight() const               { return getValue1();}
            /// Set Pulse Height for version > 0 == Set Value1
            bool     setPulseHeight (const uint32_t ph)   { return setValue1(ph);}

            /// Set method for all parameters of the NanoSlice
            bool     setParameters (const NanoSlice&);


            /// Check if the reserved word is 0xcafe is not allowed for version 1, because there is no Reserved field
            bool     checkReservedForCafe() const     { return GENERATE_FUNCTION_IS_NOT_ALLOWED;};

            IMPLEMENT_FIRST_GET_AND_SET(Value1, uint32_t)
                    
  };// end of RawNanoSlice
  
  
}} // END NAMESPCE daqdataformats
#endif /* RAWNANOSLICEV1_H */
