#ifndef _DBUTILS_DAQCONFIG_DAQMONITORRUNCONFIG_H
#define _DBUTILS_DAQCONFIG_DAQMONITORRUNCONFIG_H

#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching of NDM (NDM) run configuration data.
 *
 * @author S. Kasahara
 * @version $Revision: 1.1 $ $Date: 2014/08/06 20:29:01 $
 */

class NDMRunConfig {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    NDMRunConfig(const std::vector<int64_t>& subsystemConfigIdList);

    NDMRunConfig(const SubsystemNameSpec& nameSpec);

    NDMRunConfig(const GlobalNameSpec& nameSpec);

    NDMRunConfig(const int64_t& globalConfigId);

    virtual ~NDMRunConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}
    std::string getRRDRootDir() const { return _rrdRootDir; }
    std::string getThresholdDir() const { return _thresholdDir; }


 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    std::string _rrdRootDir; // root directory of rrd database 
    std::string _thresholdDir; // directory where thresholds are specified
};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
