#ifndef _NOVADAQ_CACHE_MANAGER_H
#define _NOVADAQ_CACHE_MANAGER_H

#include <NovaDAQUtilities/Cache.h>
#include <boost/thread/mutex.hpp>
#include <vector>

namespace novadaq {

/**
 * A singleton class to manage flushing of caches.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2009/11/18 14:12:04 $
 */

class CacheManager {

 public:

    static CacheManager& getInstance();

    void registerCache(Cache* cache);

    void removeCache(Cache* cache);

    void flushCaches();

 private:

    /**
     * Default constructor.
     */
    explicit CacheManager() {};

    // disallow any copying
    CacheManager(CacheManager const&);  // not implemented
    void operator= (CacheManager const&);  // not implemented

    /**
     * The list of registered caches.
     */
    std::vector<Cache*> _registry;

    /**
     * A mutex to protect against concurrent use of the registry.
     */
    boost::mutex _registryMutex;

};

};  // end of namespace novadaq

#endif
