#include "DAQDataFormats/RawDataBlockHeaderV0.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlockHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE datablockheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------------------------------------------------------------
// Default constructor
RawDataBlockHeader::RawDataBlockHeader(const version_t version):
    RawDAQData(version)
{ }

//------------------------------------------------------------------------------
RawDataBlockHeader::RawDataBlockHeader(const RawDataBlockHeader& copyin):
     RawDAQData(copyin)
    ,DEEPCOPY_GET_AND_SET(Version)
    ,DEEPCOPY_GET_AND_SET(Marker)
    ,DEEPCOPY_GET_AND_SET(TrigNumLo)
    ,DEEPCOPY_GET_AND_SET(TrigNumHi)
    ,DEEPCOPY_GET_AND_SET(NumMicroBlocks)
    ,DEEPCOPY_GET_AND_SET(BuffId)
    ,DEEPCOPY_GET_AND_SET(DataSize)
    ,DEEPCOPY_GET_AND_SET(MonteCarloFlag)
    ,DEEPCOPY_IS_AND_SET(CRCCalculationUsed)
    ,DEEPCOPY_IS_AND_SET(ConnectionInitialization)
    ,DEEPCOPY_IS_AND_SET(CloseConnection)
    ,DEEPCOPY_IS_AND_SET(MissingData)
    ,DEEPCOPY_GET_AND_SET(TrigNum)
    ,DEEPCOPY_FUNCTION(addDataSize)
    ,DEEPCOPY_FUNCTION(advanceNumMicroBlocks)
    ,DEEPCOPY_FUNCTION(checkMarker)
{ }

//------------------------------------------------------------------------------
// Destructor
RawDataBlockHeader::~RawDataBlockHeader() { }

//------------------------------------------------------------------------------
// Initialization of data members
void RawDataBlockHeader::init(){

    // Set the Marker
    EXECUTE_FUNCTION(setVersion) (_version);
    EXECUTE_FUNCTION(setMarker)  (daqdataformats::datablockheader::DataBlock_Marker);

    /// In the beginning of the datablock we havn't calculated CRC yet
    /// So, set it to false
    EXECUTE_FUNCTION(setCRCCalculationUsed)(false);

    /// Size of the header + CRC
    EXECUTE_FUNCTION(setDataSize)(EXECUTE_FUNCTION(sizeofdata)() + 1);
}

//------------------------------------------------------------------------------
// Getters
//------------------------------------------------------------------------------
uint64_t RawDataBlockHeader::getTrigNum() const { 
      return getUint64_t(EXECUTE_FUNCTION(getTrigNumLo)(), EXECUTE_FUNCTION(getTrigNumHi)());
}

//------------------------------------------------------------------------------
bool RawDataBlockHeader::setTrigNum(const uint64_t trigstamp) {
      uint32_t low, high;
      getLoAndHi(trigstamp, low, high);

      EXECUTE_FUNCTION(setTrigNumLo)(low);
      RETURN_FUNCTION(setTrigNumHi)(high);
}

//------------------------------------------------------------------------------
bool RawDataBlockHeader::addDataSize  (const uint32_t size){ RETURN_FUNCTION(setDataSize)(EXECUTE_FUNCTION(getDataSize)() + size);}
bool RawDataBlockHeader::advanceNumMicroBlocks()           { RETURN_FUNCTION(setNumMicroBlocks)(EXECUTE_FUNCTION(getNumMicroBlocks)() + 1);}
bool RawDataBlockHeader::checkMarker()       const         { return (EXECUTE_FUNCTION(getMarker)() == daqdataformats::datablockheader::DataBlock_Marker);}

//------------------------------------------------------------------------------
// Print the Data Block Header
void RawDataBlockHeader::print(std::ostream& os) const{

    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<boost::format("\nRawDataBlockHeader Header (size=%u, version=%u):  ") % size_of_data % _version<<std::endl;
    printBuffer(os);

    os<<boost::format("Version=%u"                   ) % EXECUTE_FUNCTION(getVersion)();
    os<<boost::format(" Marker=%u"                    ) % EXECUTE_FUNCTION(getMarker)();
    os<<boost::format(" TriggerNumberLo=%u"           ) % EXECUTE_FUNCTION(getTrigNumLo)();
    os<<boost::format(" TriggerNumberHi=%u"           ) % EXECUTE_FUNCTION(getTrigNumHi)();
    os<<boost::format(" NumMicroBlocks=%u"            ) % EXECUTE_FUNCTION(getNumMicroBlocks)();
    os<<boost::format(" BuffId=%u"                    ) % EXECUTE_FUNCTION(getBuffId)();
    os<<boost::format(" DataSize=%u"                  ) % EXECUTE_FUNCTION(getDataSize)();
    os<<boost::format(" MonteCarloFlag=%u"            ) % EXECUTE_FUNCTION(getMonteCarloFlag)();
    os<<boost::format(" isCRCCalculationUsed=%u"      ) % EXECUTE_FUNCTION(isCRCCalculationUsed)();
    os<<boost::format(" isConnectionInitialization=%u") % EXECUTE_FUNCTION(isConnectionInitialization)();
    os<<boost::format(" isCloseConnection=%u"         ) % EXECUTE_FUNCTION(isCloseConnection)();
    os<<boost::format(" isMissingData=%u"             ) % EXECUTE_FUNCTION(isMissingData)();
    os<<std::endl;

    os<<"\nRawDataBlockHeader RAW bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<boost::format("\n");
        printWord(i, true, os);
    }

}// end of print


}}// end of namespaces







      

