#include "DAQDataFormats/RawTriggerTimeV0.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerTime
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggertime
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------Constructors------------------------------------------
RawTriggerTime::RawTriggerTime(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(Version       )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStampLow  )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStampHigh )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStamp_sec )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStamp_usec)
            ,ZEROOUT_GET_AND_SET(ClockSource          )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStamp      )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStampTvUsec)
            ,ZEROOUT_GET_AND_SET(TriggerTimeStampTvSec )
            ,ZEROOUT_GET_AND_SET(TriggerTimeStampTv    )
{}

//------------------------------------------------------------------------------
RawTriggerTime::RawTriggerTime(const RawTriggerTime& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(Version       )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStampLow  )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStampHigh )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStamp_sec )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStamp_usec)
            ,DEEPCOPY_GET_AND_SET(ClockSource          )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStamp      )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStampTvUsec)
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStampTvSec )
            ,DEEPCOPY_GET_AND_SET(TriggerTimeStampTv    )
{}

//------------------------------------------------------------------------------
RawTriggerTime::~RawTriggerTime(){}

//------------------------------------------------------------------------------
// General Initialization
void RawTriggerTime::init(){
    EXECUTE_FUNCTION(setVersion)(_version);
}

//------------------------------------------------------------------------------
// get Method for 64 bit time stamp
uint64_t RawTriggerTime::getTriggerTimeStamp() const          { 
    return getUint64_t(EXECUTE_FUNCTION(getTriggerTimeStampLow)(), EXECUTE_FUNCTION(getTriggerTimeStampHigh)());
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp
bool RawTriggerTime::setTriggerTimeStamp(const uint64_t timestamp)  {
      uint32_t low, high;
      getLoAndHi(timestamp, low, high);

      EXECUTE_FUNCTION(setTriggerTimeStampHigh)(high);
      RETURN_FUNCTION(setTriggerTimeStampLow)(low);
}

//------------------------------------------------------------------------------
// get Method for the the seconds of the timeval
time_t RawTriggerTime::getTriggerTimeStampTvSec()       const { return (time_t) EXECUTE_FUNCTION(getTriggerTimeStamp_sec)();}
//------------------------------------------------------------------------------
// Get Method for the the microseconds of the timeval
suseconds_t RawTriggerTime::getTriggerTimeStampTvUsec() const { return (suseconds_t) EXECUTE_FUNCTION(getTriggerTimeStamp_usec)();}
//------------------------------------------------------------------------------
bool RawTriggerTime::setTriggerTimeStampTvSec (const time_t      a) { RETURN_FUNCTION(setTriggerTimeStamp_sec)((const uint32_t)a);}
bool RawTriggerTime::setTriggerTimeStampTvUsec(const suseconds_t a) { RETURN_FUNCTION(setTriggerTimeStamp_usec)((const uint32_t)a);}

//------------------------------------------------------------------------------
// get Method for 64 bit time stamp, in timeval format
timeval RawTriggerTime::getTriggerTimeStampTv() const {
  timeval timestamp;
  timestamp.tv_usec = EXECUTE_FUNCTION(getTriggerTimeStampTvUsec)();
  timestamp.tv_sec  = EXECUTE_FUNCTION(getTriggerTimeStampTvSec)();

  return timestamp;
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp, as a timeval
bool RawTriggerTime::setTriggerTimeStampTv(timeval timestamp){

    EXECUTE_FUNCTION(setTriggerTimeStampTvUsec)(timestamp.tv_usec);
    RETURN_FUNCTION(setTriggerTimeStampTvSec)(timestamp.tv_sec);
}

//------------------------------------------------------------------------------
void RawTriggerTime::print(std::ostream& os) const{
    os<<"++++ Trigger Time ++++\n";
    os<<boost::format("Trigger Time Version:\t%u\n")%EXECUTE_FUNCTION(getVersion)();
    os<<boost::format("Gen Timestamp Low:\t%u\n")%EXECUTE_FUNCTION(getTriggerTimeStampLow)();
    os<<boost::format("Gen Timestamp High:\t%u\n")%EXECUTE_FUNCTION(getTriggerTimeStampHigh)();
    os<<boost::format("Gen Timestamp:\t\t%Lu\n")%(long long unsigned int)EXECUTE_FUNCTION(getTriggerTimeStamp)();
    time_t TrigSec = EXECUTE_FUNCTION(getTriggerTimeStampTvSec)();
    char ctimestr[27]; ctime_r(&TrigSec,ctimestr);
    os<<boost::format("Gen Time:\t\t%s")%ctimestr;
    os<<boost::format("Gen usec:\t\t%d\n")%(uint32_t) EXECUTE_FUNCTION(getTriggerTimeStampTvUsec)();
    os<<boost::format("Clock Source:\t\t0x%02x\n")%EXECUTE_FUNCTION(getClockSource)();
    os<<boost::format("TriggerTime Version:\t%d\n")%EXECUTE_FUNCTION(getVersion)();
    PRINT_ON_DEBUG("Buffer is set to "<<_Buffer)

}

}}// end of namespaces