#include "DAQDataFormats/RawTriggerTimingMarkerV0.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerTimingMarker
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggertimingmarker
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------Constructors------------------------------------------
RawTriggerTimingMarker::RawTriggerTimingMarker(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(Version       )
            ,ZEROOUT_GET_AND_SET(TriggerStartLow_baseClock)
            ,ZEROOUT_GET_AND_SET(TriggerStartHigh_baseClock)
            ,ZEROOUT_GET_AND_SET(TriggerStartLow_833kHz    )
            ,ZEROOUT_GET_AND_SET(TriggerStartHigh_833kHz   )
            ,ZEROOUT_GET_AND_SET(TriggerStart_baseClock)
            ,ZEROOUT_GET_AND_SET(TriggerStart_833kHz   )
{}

//------------------------------------------------------------------------------
RawTriggerTimingMarker::RawTriggerTimingMarker(const RawTriggerTimingMarker& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(Version       )
            ,DEEPCOPY_GET_AND_SET(TriggerStartLow_baseClock)
            ,DEEPCOPY_GET_AND_SET(TriggerStartHigh_baseClock)
            ,DEEPCOPY_GET_AND_SET(TriggerStartLow_833kHz    )
            ,DEEPCOPY_GET_AND_SET(TriggerStartHigh_833kHz   )
            ,DEEPCOPY_GET_AND_SET(TriggerStart_baseClock)
            ,DEEPCOPY_GET_AND_SET(TriggerStart_833kHz   )
{}

//------------------------------------------------------------------------------
RawTriggerTimingMarker::~RawTriggerTimingMarker(){}

//------------------------------------------------------------------------------
// General Initialization
void RawTriggerTimingMarker::init(){
    EXECUTE_FUNCTION(setVersion)(_version);
}

//------------------------------------------------------------------------------
// get Method for 64 bit start time on the base clock
uint64_t RawTriggerTimingMarker::getTriggerStart_baseClock() const         { 
    return getUint64_t(EXECUTE_FUNCTION(getTriggerStartLow_baseClock)(), EXECUTE_FUNCTION(getTriggerStartHigh_baseClock)());
}

//------------------------------------------------------------------------------
// get Method for 64 bit start time on the base clock
uint64_t RawTriggerTimingMarker::getTriggerStart_833kHz()    const         { 
    return getUint64_t(EXECUTE_FUNCTION(getTriggerStartLow_833kHz)(), EXECUTE_FUNCTION(getTriggerStartHigh_833kHz)());
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp
bool RawTriggerTimingMarker::setTriggerStart_baseClock(const uint64_t timestamp) {
      uint32_t low, high;
      getLoAndHi(timestamp, low, high);

      EXECUTE_FUNCTION(setTriggerStartHigh_baseClock)(high);
      RETURN_FUNCTION(setTriggerStartLow_baseClock)(low);
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp
bool RawTriggerTimingMarker::setTriggerStart_833kHz   (const uint64_t timestamp) {

      uint32_t low, high;
      getLoAndHi(timestamp, low, high);

      EXECUTE_FUNCTION(setTriggerStartHigh_833kHz)(high);
      RETURN_FUNCTION(setTriggerStartLow_833kHz)(low);
}


//------------------------------------------------------------------------------
void RawTriggerTimingMarker::print(std::ostream& os) const{

    os<<"++++ Trigger Timing Marker ++++\n";

    printBuffer(os);

    os<<boost::format("Trigger Timing Marker Version:\t%u\n")%EXECUTE_FUNCTION(getVersion)();
    os<<boost::format("Start Low Nova Clock:\t\t%u\n")%EXECUTE_FUNCTION(getTriggerStartLow_baseClock)();
    os<<boost::format("Start High Nova Clock:\t\t%u\n")%EXECUTE_FUNCTION(getTriggerStartHigh_baseClock)();
    os<<boost::format("Full Nova Clock:\t\t%Lu\n")%(long long unsigned int)EXECUTE_FUNCTION(getTriggerStart_baseClock)();
    //os<<boost::format("Start Time:\t\t%s\n")%novadaq::timeutils::convertNovaTimeToString(getTriggerStart_baseClock()).c_str();
    os<<boost::format("Extraction Start Low:\t\t%u\n")%EXECUTE_FUNCTION(getTriggerStartLow_833kHz)();
    os<<boost::format("Extraction Start High:\t\t%u\n")%EXECUTE_FUNCTION(getTriggerStartHigh_833kHz)();

}

}}// end of namespaces
