//  This file (compat_reader.cc) was created by Ron Rechenmacher <ron@fnal.gov> on
//  Sep 27, 2013. "TERMS AND CONDITIONS" governing this file are in the README
//  or COPYING file. If you do not have such a file, one can be obtained by
//  contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
//  $RCSfile: compat_reader.cc,v $
//  rev="$Revision: 1.3 $$Date: 2015/07/16 14:55:35 $";

#ifdef DARWINBUILD
 #include <libgen.h>
#endif
#include <stdlib.h>		// malloc/free
#include <string.h>		// memcpy
#include <getopt.h>		// getopt_long
#include "../include/ShmRdWr.h"

#define DFLT_SLEEP_US	500000
#define USAGE "\
   Usage: %s [opts]\n\
Examples: %s -s500000 # .5 second sleep\n\
 Options: -s<sleep_us>  (dflt:%d)\n\
          -? -h --help  help/usage\n\
",      basename(argv[0]), basename(argv[0]),\
	DFLT_SLEEP_US

int
main(  int	argc
     , char	*argv[] )
{
    unsigned long opt_sleep_us=DFLT_SLEEP_US;
    while (1)
    {   int                  c, option_index=0;
        static struct option long_options[]=
            {   // name,    has_arg,flag,val
                {"sleep",    1,     0,  's'},
		{"help",     0,     0,  'h'},
                {0, 0, 0, 0}
            };
        //                            +=stop parsing at first non-option
        c = getopt_long( argc, argv, "+?hs:", long_options, &option_index );
        if (c == -1)  break;
        switch (c)
        {
        case '?': case 'h': printf(USAGE); exit(0);    break;
        case 's': opt_sleep_us=strtoul(optarg,NULL,0); break;
        default:printf("?? getopt returned character code %d (0x%x) ??\n",c,c);
        }
    }


    ShmRd   sharedMemoryReader_;
    shm_rdwr_buffer_header   *   hdr;
    ShmRdWrCnt          missed;
    uint32_t          cushion;
    int           new_tmo = 1000000;
    uint32_t      counter=0;

    while (1)
    {   printf( "start_read %u\n",counter++ );

	int shm_read_status =
	    sharedMemoryReader_.start_read(&hdr, missed, cushion, new_tmo);

	if (shm_read_status <= 0)
	{   printf( "\nNO DATA AVAILABLE - missed=%u\n\n", missed );
	    usleep( 5000000 );
	    continue;
	}

	size_t nbytesInPayload = hdr->bytes_total;
	shm_rdwr_buffer_header * afterTheShmHeader = hdr + 1;
	printf("nbytesInPayload=%ld\n", nbytesInPayload );

	uint8_t *dest=(uint8_t*)malloc(nbytesInPayload);
	memcpy( dest, &*afterTheShmHeader, nbytesInPayload);
	free(dest);

	bool overwrite_happened = // We've scribbled on the event.
	    (sharedMemoryReader_.check_overwrite(missed, cushion) != 0);
	printf("%d=overwrite_happened missed=%u\n",overwrite_happened,missed);
	sharedMemoryReader_.end_read(missed, cushion);
	printf("end_read - missed=%u\n", missed );

	usleep( opt_sleep_us );
    }
    return (0);
}   // main
